# Generated by Django 2.1.14 on 2019-11-29 14:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0030_task_phase'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='issuetype',
            options={'ordering': ('label',)},
        ),
        migrations.AlterModelOptions(
            name='licensetype',
            options={'ordering': ('label',)},
        ),
        migrations.AlterModelOptions(
            name='phase',
            options={'ordering': ('title',)},
        ),
        migrations.AlterModelOptions(
            name='priority',
            options={'ordering': ('sort_order',), 'verbose_name_plural': 'Priorities'},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'ordering': ('name',)},
        ),
        migrations.AlterModelOptions(
            name='projecttype',
            options={'ordering': ('label',)},
        ),
        migrations.AlterModelOptions(
            name='source',
            options={'ordering': ('label',)},
        ),
        migrations.AlterModelOptions(
            name='subissuetype',
            options={'ordering': ('label',)},
        ),
        migrations.AlterModelOptions(
            name='ticketcategory',
            options={'ordering': ('name',), 'verbose_name_plural': 'Ticket categories'},
        ),
        migrations.AlterModelOptions(
            name='tickettype',
            options={'ordering': ('label',)},
        ),
    ]
