# Generated by Django 2.1.11 on 2019-09-20 14:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0005_auto_20190916_1549'),
        ('djautotask', '0005_remove_ticketstatus_value'),
    ]

    operations = [
    ]
