# Generated by Django 2.1.14 on 2020-01-31 10:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0038_notetype_tasknote_ticketnote'),
        ('djautotask', '0038_auto_20200130_1218'),
    ]

    operations = [
    ]
