# Generated by Django 2.1.11 on 2020-05-14 17:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0066_auto_20200512_1420'),
        ('djautotask', '0064_servicecall_location'),
    ]

    operations = [
    ]
